//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Cylinder.h"
#include "MathsHelp.h"
#include "StringHelp.h"
#include <clipbrd.hpp>
#include <IniFiles.hpp>
#include <math.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TCylinderForm *CylinderForm;
//---------------------------------------------------------------------------
__fastcall TCylinderForm::TCylinderForm(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------

void TCylinderForm::UpdateGCode(void)
{
	Memo1->Lines->Clear();

	//read data
	int linenumberincrement = LineNumberIncrement->Text.ToIntDef(1);
	if (linenumberincrement < 1) linenumberincrement = 1;
	int currentlinenumber = linenumberincrement;
	String text;

	double waypointX = ToDoubleDef(WaypointX->Text,0.0);
	double waypointZ = ToDoubleDef(WaypointZ->Text,0.0);
	double startpointX = ToDoubleDef(StartpointX->Text,0.0);
	double startpointZ = ToDoubleDef(StartpointZ->Text,0.0);
	double endpointX = ToDoubleDef(EndpointX->Text,0.0);
	double endpointZ = ToDoubleDef(EndpointZ->Text,0.0);
	double clearanceX = fabs(ToDoubleDef(ClearanceX->Text,1.0));
	double clearanceZ = fabs(ToDoubleDef(ClearanceZ->Text,1.0));
	double feedrate = ToDoubleDef(Feedrate->Text,1.0);
	double roughingdoc = fabs(ToDoubleDef(RoughingDOC->Text,0.5));
	double finishdoc = fabs(ToDoubleDef(FinishDOC->Text,0.05));
	double endoffset = finishdoc;
	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "(Start Point X" + FormatFloat("####0.0000",startpointX) + " Z" + FormatFloat("####0.0000",startpointZ) + ")";
	Memo1->Lines->Add(text);
	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "(End Point X" + FormatFloat("####0.0000",endpointX) + " Z" + FormatFloat("####0.0000",endpointZ) + ")";
	Memo1->Lines->Add(text);
	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "(Clearances X" + FormatFloat("####0.00##",clearanceX) + " Z" + FormatFloat("####0.00##",clearanceZ) + ")";
	Memo1->Lines->Add(text);
	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "(Depth of Cut    Roughing:" + FormatFloat("####0.00##",roughingdoc) + "    Finish:" + FormatFloat("####0.00##",finishdoc) + ")";
	Memo1->Lines->Add(text);


	//now to check orientation of cuts for turning/boring to/from headstock
	if (startpointX < endpointX) {
		if (clearanceX > 0.0) clearanceX = clearanceX * -1.0;
		if (roughingdoc > 0.0) roughingdoc = roughingdoc * -1.0;
		if (finishdoc > 0.0) finishdoc = finishdoc * -1.0;
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "(Boring)";
		Memo1->Lines->Add(text);
		}
	if (startpointZ < endpointZ && clearanceZ > 0.0) {
		clearanceZ = clearanceZ * -1.0;
		endoffset = endoffset * -1.0;
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "(Cutting towards tailstock)";
		Memo1->Lines->Add(text);
		}

	double lastroughingX = endpointX + finishdoc;
	int numberofloops = (int)((startpointX - lastroughingX)/roughingdoc);
	double lastroughingdia = (startpointX - ((double)(numberofloops))*roughingdoc);
	double lastroughingdoc = lastroughingdia - lastroughingX;
		//debug
		/*
		Memo1->Lines->Add("startpointX = " + String(startpointX));
		Memo1->Lines->Add("numberofloops-1 = " + String(numberofloops-1));
		Memo1->Lines->Add("roughingdoc = " + String(roughingdoc));
		Memo1->Lines->Add("lastroughingdia = " + String(lastroughingdia));
		Memo1->Lines->Add("lastroughingdoc = " + String(lastroughingdoc));
        */
		//end debug
	/*if (lastroughingdoc < 0.0) {
		Memo1->Lines->Add("Diameter error");
		}*/
	bool lastrough = true;
	if (lastroughingdoc == 0.0) lastrough = false;
	if (lastroughingdoc == roughingdoc) {
		numberofloops++;
		lastrough = false;
		}



	//Add the Header text to the program file
	if (Header->Lines->Count > 0) {
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "(Header)";
		Memo1->Lines->Add(text);
		for (int i = 0; i < Header->Lines->Count; i++) {
			text = "";
			if (NumberLines->Checked) {
				text = "N" + String(currentlinenumber) + " ";
				currentlinenumber += linenumberincrement;
				}
			text += Header->Lines->Strings[i].UpperCase();
			Memo1->Lines->Add(text);
			}
		}

	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "(Set Absolute Units, Set Units, Set Diameter/Radius Mode, Set Feedrate Mode)";
	Memo1->Lines->Add(text);
	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "G90 ";
	if (Units->ItemIndex == 0) {
		text += "G21 ";
		} else {
		text += "G20 ";
		}
	if (MeasurementMode->ItemIndex == 0) {
		text += "G7 ";
		} else {
		text += "G8 ";
		}
	if (FeedRateMode->ItemIndex == 0) {
		text += "G94 ";
		} else {
		text += "G95 ";
		}
	Memo1->Lines->Add(text);

	//Waypoint
	if (Waypoint->Checked) {
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "(Rapid Move to waypoint to avoid work/tooling)";
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",waypointX) + " Z" + FormatFloat("####0.0000",waypointZ);
		Memo1->Lines->Add(text);
		}

	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "(Rapid Move to start point + X Clearance + Z Clearance)";
	Memo1->Lines->Add(text);
	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "G00 X" + FormatFloat("####0.0000",startpointX + clearanceX) + " Z" + FormatFloat("####0.0000",startpointZ + clearanceZ);
	Memo1->Lines->Add(text);

	//set the feedrate
	text = "";
	if (NumberLines->Checked) {
		text = "N" + String(currentlinenumber) + " ";
		currentlinenumber += linenumberincrement;
		}
	text += "F" + FormatFloat("####0.####",feedrate);
	Memo1->Lines->Add(text);

	//now the GCode Loop
	for (int i = 0; i < numberofloops + 1; i++) {
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "(Roughing Loop " + String(i+1) + " )";
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",startpointX - ((double)(i)) * roughingdoc) + " Z" + FormatFloat("####0.0000",startpointZ + clearanceZ);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		//text += "G01 X" + FormatFloat("####0.0000",startpointX - ((double)(i)) * roughingdoc) + " Z" + FormatFloat("####0.0000",endpointZ + finishdoc);
		text += "G01 X" + FormatFloat("####0.0000",startpointX - ((double)(i)) * roughingdoc) + " Z" + FormatFloat("####0.0000",endpointZ + endoffset);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",startpointX - ((double)(i)) * roughingdoc + clearanceX) + " Z" + FormatFloat("####0.0000",endpointZ + finishdoc + clearanceZ);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",startpointX - ((double)(i)) * roughingdoc + clearanceX) + " Z" + FormatFloat("####0.0000",startpointZ + clearanceZ);
		Memo1->Lines->Add(text);
		}
	if (lastrough) {
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "(Roughing Loop " + String(numberofloops + 2) + " )";
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",startpointX - ((double)(numberofloops)) * roughingdoc - lastroughingdoc) + " Z" + FormatFloat("####0.0000",startpointZ + clearanceZ);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		//text += "G01 X" + FormatFloat("####0.0000",startpointX - ((double)(numberofloops)) * roughingdoc - lastroughingdoc) + " Z" + FormatFloat("####0.0000",endpointZ + finishdoc);
		text += "G01 X" + FormatFloat("####0.0000",startpointX - ((double)(numberofloops)) * roughingdoc - lastroughingdoc) + " Z" + FormatFloat("####0.0000",endpointZ + endoffset);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",startpointX - ((double)(numberofloops)) * roughingdoc - lastroughingdoc + clearanceX) + " Z" + FormatFloat("####0.0000",endpointZ + finishdoc + clearanceZ);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",startpointX - ((double)(numberofloops)) * roughingdoc - lastroughingdoc + clearanceX) + " Z" + FormatFloat("####0.0000",startpointZ + clearanceZ);
		Memo1->Lines->Add(text);
		}

	//finishing cut
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "(Finishing Cut)";
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",endpointX) + " Z" + FormatFloat("####0.0000",startpointZ + clearanceZ);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G01 X" + FormatFloat("####0.0000",endpointX) + " Z" + FormatFloat("####0.0000",endpointZ);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G01 X" + FormatFloat("####0.0000",startpointX + clearanceX) + " Z" + FormatFloat("####0.0000",endpointZ);
		Memo1->Lines->Add(text);
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "G00 X" + FormatFloat("####0.0000",startpointX + clearanceX) + " Z" + FormatFloat("####0.0000",startpointZ + clearanceZ);
		Memo1->Lines->Add(text);

	//Add the Footer text to the program file
	if (Footer->Lines->Count > 0) {
		text = "";
		if (NumberLines->Checked) {
			text = "N" + String(currentlinenumber) + " ";
			currentlinenumber += linenumberincrement;
			}
		text += "(Footer)";
		Memo1->Lines->Add(text);
		for (int i = 0; i < Footer->Lines->Count; i++) {
			text = "";
			if (NumberLines->Checked) {
				text = "N" + String(currentlinenumber) + " ";
				currentlinenumber += linenumberincrement;
				}
			text += Footer->Lines->Strings[i].UpperCase();
			Memo1->Lines->Add(text);
			}
		}

	//Return focus to top of memo list
    //ActiveControl = Memo1;
	//Memo1->SelLength = 0;
	Memo1->SelStart = 0;
	Memo1->SelLength = 1;
	Memo1->SelLength = 0;
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::FormCreate(TObject *Sender)
{
	//Read settings & put them here
	TIniFile *IniFile = new TIniFile(ExtractFileDir(Application->ExeName) + "\\Data.ini");

	NumberLines->Checked = IniFile->ReadBool("LatheCylinder","NumberLines",false);
	LineNumberIncrement->Text = IniFile->ReadString("LatheCylinder","LineNumberIncrement","10");
	Units->ItemIndex = IniFile->ReadInteger("LatheCylinder","Units",0);
	MeasurementMode->ItemIndex = IniFile->ReadInteger("LatheCylinder","MeasurementMode",0);
	FeedRateMode->ItemIndex = IniFile->ReadInteger("LatheCylinder","FeedRateMode",0);
	Waypoint->Checked = IniFile->ReadBool("LatheCylinder","Waypoint",false);
	WaypointX->Text = IniFile->ReadString("LatheCylinder","WaypointX","0.0");
	WaypointZ->Text = IniFile->ReadString("LatheCylinder","WaypointZ","0.0");
	StartpointX->Text = IniFile->ReadString("LatheCylinder","StartpointX","0.0");
	StartpointZ->Text = IniFile->ReadString("LatheCylinder","StartpointZ","0.0");
	EndpointX->Text = IniFile->ReadString("LatheCylinder","EndpointX","0.0");
	EndpointZ->Text = IniFile->ReadString("LatheCylinder","EndpointZ","0.0");
	ClearanceX->Text = IniFile->ReadString("LatheCylinder","ClearanceX","1.0");
	ClearanceZ->Text = IniFile->ReadString("LatheCylinder","ClearanceZ","1.0");
	Feedrate->Text = IniFile->ReadString("LatheCylinder","Feedrate","1.0");
	RoughingDOC->Text = IniFile->ReadString("LatheCylinder","RoughingDOC","0.5");
	FinishDOC->Text = IniFile->ReadString("LatheCylinder","FinishDOC","0.05");

	int numberoflines = IniFile->ReadInteger("LatheCylinder","NumberofHeaderLines",0);
	String text;
	for (int i = 0; i < numberoflines; i++) {
		text = "HeaderLine" + String(i);
		Header->Lines->Add(IniFile->ReadString("LatheCylinder",text," "));
		}
	numberoflines = IniFile->ReadInteger("LatheCylinder","NumberofFooterLines",0);
	for (int i = 0; i < numberoflines; i++) {
		text = "FooterLine" + String(i);
		Footer->Lines->Add(IniFile->ReadString("LatheCylinder",text," "));
		}

	delete IniFile;

	//format screen
	NumberLinesClick(Sender);
	WaypointClick(Sender);

	//Update code generated
	UpdateGCode();
	//Now activate the OnChange events
	NumberLines->OnClick = NumberLinesClick;
	LineNumberIncrement->OnChange = UnitsClick;
	Units->OnClick = UnitsClick;
	MeasurementMode->OnClick = UnitsClick;
	FeedRateMode->OnClick = UnitsClick;
	Waypoint->OnClick = WaypointClick;
	WaypointX->OnChange = UnitsClick;
	WaypointZ->OnChange = UnitsClick;
	StartpointX->OnChange = UnitsClick;
	StartpointZ->OnChange = UnitsClick;
	EndpointX->OnChange = UnitsClick;
	EndpointZ->OnChange = UnitsClick;
	ClearanceX->OnChange = ClearanceXChange;
	ClearanceZ->OnChange = UnitsClick;
	Feedrate->OnChange = UnitsClick;
	RoughingDOC->OnChange = UnitsClick;
	FinishDOC->OnChange = UnitsClick;
	Header->OnChange = UnitsClick;
	Footer->OnChange = UnitsClick;
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::NumberLinesClick(TObject *Sender)
{
	Label5->Enabled = NumberLines->Checked;
	LineNumberIncrement->Enabled = NumberLines->Checked;
	if (LineNumberIncrement->Enabled) {
		LineNumberIncrement->Color = clWindow;
		} else {
		LineNumberIncrement->Color = clBtnFace;
		}
	//Update code generated
	UpdateGCode();
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::UnitsClick(TObject *Sender)
{
	//Update code generated
	UpdateGCode();
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::WaypointClick(TObject *Sender)
{
	Label1->Enabled = Waypoint->Checked;
	WaypointX->Enabled = Waypoint->Checked;
	WaypointZ->Enabled = Waypoint->Checked;
	if (WaypointX->Enabled) {
		WaypointX->Color = clWindow;
		WaypointZ->Color = clWindow;
		} else {
		WaypointX->Color = clBtnFace;
		WaypointZ->Color = clBtnFace;
		}
	//Update code generated
	UpdateGCode();
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::ClearanceXChange(TObject *Sender)
{
    ClearanceZ->Text = ClearanceX->Text;
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::FormDestroy(TObject *Sender)
{
	//Read settings & put them here
	TIniFile *IniFile = new TIniFile(ExtractFileDir(Application->ExeName) + "\\Data.ini");

    IniFile->EraseSection("LatheCylinder");
	IniFile->WriteBool("LatheCylinder","NumberLines",NumberLines->Checked);
	IniFile->WriteString("LatheCylinder","LineNumberIncrement",LineNumberIncrement->Text);
	IniFile->WriteInteger("LatheCylinder","Units",Units->ItemIndex);
	IniFile->WriteInteger("LatheCylinder","MeasurementMode",MeasurementMode->ItemIndex);
	IniFile->WriteInteger("LatheCylinder","FeedRateMode",FeedRateMode->ItemIndex);
	IniFile->WriteBool("LatheCylinder","Waypoint",Waypoint->Checked);
	IniFile->WriteString("LatheCylinder","WaypointX",WaypointX->Text);
	IniFile->WriteString("LatheCylinder","WaypointZ",WaypointZ->Text);
	IniFile->WriteString("LatheCylinder","StartpointX",StartpointX->Text);
	IniFile->WriteString("LatheCylinder","StartpointZ",StartpointZ->Text);
	IniFile->WriteString("LatheCylinder","EndpointX",EndpointX->Text);
	IniFile->WriteString("LatheCylinder","EndpointZ",EndpointZ->Text);
	IniFile->WriteString("LatheCylinder","ClearanceX",ClearanceX->Text);
	IniFile->WriteString("LatheCylinder","ClearanceZ",ClearanceZ->Text);
	IniFile->WriteString("LatheCylinder","Feedrate",Feedrate->Text);
	IniFile->WriteString("LatheCylinder","RoughingDOC",RoughingDOC->Text);
	IniFile->WriteString("LatheCylinder","FinishDOC",FinishDOC->Text);

	IniFile->WriteInteger("LatheCylinder","NumberofHeaderLines",Header->Lines->Count);
	String text;
	for (int i = 0; i < Header->Lines->Count; i++) {
		text = "HeaderLine" + String(i);
		IniFile->WriteString("LatheCylinder",text,Header->Lines->Strings[i]);
		}
	IniFile->WriteInteger("LatheCylinder","NumberofFooterLines",Footer->Lines->Count);
	for (int i = 0; i < Footer->Lines->Count; i++) {
		text = "FooterLine" + String(i);
		IniFile->WriteString("LatheCylinder",text,Footer->Lines->Strings[i]);
		}

	delete IniFile;

}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::SaveAs1Click(TObject *Sender)
{
	//save code
	if (SaveDialog1->Execute()) {
        Memo1->Lines->SaveToFile(SaveDialog1->FileName);
		}
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::Exit1Click(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::Edit1Click(TObject *Sender)
{
	Copy1->Enabled = (Memo1->SelLength > 0);
    Paste1->Enabled = (Clipboard()->HasFormat(CF_TEXT));
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::SelectAll1Click(TObject *Sender)
{
	Memo1->SelStart = 0;
    Memo1->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::Paste1Click(TObject *Sender)
{
	Memo1->PasteFromClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TCylinderForm::Help2Click(TObject *Sender)
{

	String temp;

	temp = "This program generates G Code for a CNC lathe to cut a cylinder\n";
	temp = temp + "\n";
	temp = temp + "To operate, fill in all the data and copy/save the G Code\n";
	String title = "Help on G Code Lathe Cylinder Toolpath Generation Program";
	Application->MessageBox(temp.c_str(),
							title.c_str(),
                            MB_OK | MB_ICONINFORMATION);

}
//---------------------------------------------------------------------------

